#' ---
#' title: "GCA Analysis"
#' author: ""
#' date: ""
#' ---

#### Install and run packages ##############################################################

#install.packages("librarian")
librarian::shelf(lme4, Matrix, tidyr, ggplot2, ggpattern, readxl, tidyverse, merTools, gridExtra, beepr)

#### Prepare Data ######################################################################

#Set Working Directory in R Studio
#R Studio File and Data File should be in the same folder 
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()
dir.create("GCA Output/") #Create Directory to hold output files


#Load File and Set up data
GCAWide<- read_excel("GCA Data.xlsx")
#view(GCAWide) #Check Data if necessary
GCA<-pivot_longer(GCAWide, cols=num_range("",1:50), names_to="Time", values_to="Accuracy", values_drop_na = TRUE)
GCA$Time<-as.numeric(GCA$Time)
#view(GCA)

#Get Summary and Head
summary(GCA)
head(GCA)

#Apply filter for Experiment, Group, etc.
GCA<-filter(GCA, Treatment != "PI+P+EI", Experiment == "Henry17")
table(GCA$Treatment, GCA$Experiment)

#### Plot Raw Data ######################################################################

plot<-ggplot(GCA, aes(Time, Accuracy, color=Treatment))+stat_summary(fun=mean,geom="line")
facet<-facet_wrap(~Experiment)
plot+facet


#############################################################################################
#### Begin GCA Analysis #####################################################################
#############################################################################################

#### Prepare Data ###########################################################################

#Relevel
GCA$Treatment<-factor(GCA$Treatment, levels=c("PI-EI", "PI+EI", "PI+P-EI"))
contrasts(GCA$Treatment)

#Change to Main effects if desired; here, we'll use simple effects though
#GCA$TreatmentSum<-C(GCA$Treatment, sum)
#levels(GCA$TreatmentSum)<-c("PI-EI", "PI+EI", "PI+P-EI")
#contrasts(GCA$TreatmentSum)

#Create a fourth-order orthogonal polynomial
#Maximum polynomial suggested by Mirman (2014)
t<-poly(unique(as.numeric(GCA$Time)),4)
GCA[,paste("ot", 1:4, sep="")]<-t[as.numeric(GCA$Time), 1:4]

#Compute empirical logic
GCA$elog<-with(GCA, log((Accuracy+.5)/(1-Accuracy+.5)))
GCA$wts<-with(GCA, 1/(Accuracy+.5)+1/(1-Accuracy+.5))
head(GCA)
#view(GCA)

#############################################################################################
####Build Models Using Elogit################################################################
#############################################################################################

#Build model with 1st order polynomial#######################################################
e.1<-lmer(elog~(ot1)*Treatment+(ot1|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)

#Build model with 2nd order polynomial#######################################################
#Maximal Model
e.2a<-lmer(elog~(ot1+ot2)*Treatment+(ot1+ot2|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
#Reduce Random effects structure until singular fit warning disappears
e.2b<-lmer(elog~(ot1+ot2)*Treatment+(ot1|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)

#Build model with 3rd order polynomial#######################################################
#Maximal Model
e.3a<-lmer(elog~(ot1+ot2+ot3)*Treatment+(ot1+ot2+ot3|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
#Reduce Random effects structure until singular fit warning disappears
e.3b<-lmer(elog~(ot1+ot2+ot3)*Treatment+(ot1+ot2|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
e.3c<-lmer(elog~(ot1+ot2+ot3)*Treatment+(ot1|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)

#Build model with 4th order polynomial#######################################################
#Maximal Model
e.4a<-lmer(elog~(ot1+ot2+ot3+ot4)*Treatment+(ot1+ot2+ot3+ot4|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
#Reduce Random effects structure until singular fit warning disappears
e.4b<-lmer(elog~(ot1+ot2+ot3+ot4)*Treatment+(ot1+ot2+ot3|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
e.4c<-lmer(elog~(ot1+ot2+ot3+ot4)*Treatment+(ot1+ot2|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)
e.4d<-lmer(elog~(ot1+ot2+ot3+ot4)*Treatment+(ot1|Subject), control=lmerControl(optimizer="bobyqa"), data=GCA, weights=1/wts,REML=F)

#Compare Models that converged without singular fit
anova(e.1, e.2b, e.3c, e.4d)

#Specify Best Fit Model
BFModel<-e.3c


#############################################################################################
####Full Results and Plots ################################################################
#############################################################################################

#Get Output and add p-values ################################################################
coefs <- data.frame(coef(summary(BFModel)))
coefs$p<-format.pval(2*(1-pnorm(abs(coefs$t.value))), digits=2, eps=0.0001)
write.csv(coefs, "GCA Output/Coefs.csv")

#### Plot Results ###########################################################################

#Add the fitted values to a new data frame
GCA2<-data.frame(GCA, GCA_Full=fitted(BFModel))

#Note: If you want to see how other models compare visually to the BFModel, do something like:
#GCA2a<-data.frame(GCA, GCA_Full=fitted(e.4d))
#And then use GCA2a as your data for the prediction/model plotting below

prediction<-predictInterval(BFModel, GCA2, level=0.95, include.resid.var = FALSE)
GCA3 <- cbind(GCA2, prediction) #Add Prediction to data frame
GCA3_means <- GCA3 %>%
  group_by(Experiment, Time, Treatment) %>% #Get means for each time by Experiment and Treatment for plotting
  summarize(GCA_Full = mean(GCA_Full),
            upr = mean(upr),
            lwr = mean(lwr))
#view(GCA3)

#### Plot Color With confidence intervals ##################################################
mplot2<-filter(GCA3)%>% 
  ggplot(aes(Time, GCA_Full, group=Treatment, fill=Treatment, color=Treatment))+
  stat_summary(aes(y=GCA_Full), fun=mean, geom="line")+ #fit
  #  stat_summary(aes(y=elog), fun.data=mean_se, geom="pointrange")+
  #  stat_summary(aes(y=upr), fun=mean, geom="line", linetype="dashed")+ #dashed upper bounds
  #  stat_summary(aes(y=lwr), fun=mean, geom="line", linetype="dashed")+ #dashed lower bounds
  geom_ribbon(data=GCA3_means, aes(ymin=lwr, ymax=upr), na.rm=TRUE, color=NA , alpha = .1) + #ribbon should fit between upr and lwr dashed lines
  theme_bw(base_size=10)+facet_wrap(~Experiment) + theme(legend.position="none")
labels<-labs(x="Item", y="Accuracy (elogit)")
mplot2+labels+scale_x_continuous(breaks=seq(0,50, 10))
ggsave("GCA Output/Plot with Confidence intervals.pdf",mplot2+labels, width=16, height=12, dpi=1200)


#### Plot Black and White ##################################################################
mplotbw<-ggplot(GCA3, aes(as.numeric(Time), GCA_Full, shape=Treatment))+
#  stat_summary(fun.data=mean_se, geom="pointrange")+
  stat_summary(aes(y=GCA_Full, linetype=Treatment), fun=mean, geom="line")+
  theme_bw(base_size=20)
labelsbw<-labs(x="Item", y="Accuracy", shape="Treatment", linetype="Treatment")
mplotbw+labelsbw+facet
ggsave("GCA Output/B&W Plot.pdf",mplotbw+facet+labelsbw, width=16, height=12, dpi=1200)


#### Plot Black and White with Patterned CIs ##################################################
mplotbw<-ggplot(GCA3, aes(as.numeric(Time), GCA_Full, pattern_angle=Treatment, shape=Treatment))+
#    stat_summary(fun.data=mean_se, geom="pointrange")+
  geom_ribbon_pattern(data=GCA3_means, aes(ymin=lwr, ymax=upr), 
                      fill=NA, color=NA, pattern_spacing=.01, pattern_density=.1,
                      pattern_alpha=.25)+
                      scale_pattern_angle_manual(values=c(0, 45, 90))+
    stat_summary(aes(y=GCA_Full, linetype=Treatment), fun=mean, geom="line", linewidth=1.2)+
  theme_bw(base_size=20)
labelsbw<-labs(x="Item", y="Accuracy", shape="Treatment", linetype="Treatment")
mplotbw+labelsbw+facet
ggsave("GCA Output/B&W Plot with Pattern Ribbon.pdf",mplotbw+facet+labelsbw, width=16, height=12, dpi=1200)